/*
 *	Program do obsugi wywietlacza AFFA2++ w samochodach Renault
 *
 *  Created on: 13 lis 2018
 *      Author: Mako, Kacper
 *
 Ukad pozwalajcy na komunikacj z wywietlaczem w samochodzie, wysya do wywietlacza
 tekst do wywietlenia i odbiera nacinicia przyciskw na pilocie przy kierownicy.

 http://tlcdcemu.sourceforge.net/hardware.html
 Opis wtyczki od wywietlacza http://tlcdcemu.sourceforge.net/img/tl_pinout_anim.gif:
 C1  connector (ta, przy radiu):
 1   DTA   I2C-Data
 2   CLK   I2C-Clock
 3   MIRQ  trzeba go cign do masy przed wysyaniem
 4   n/c
 5   Radio ON	poda 5V eby si wczy wywietlacz
 6   GND

 Bdziemy uywa wszystkich pinw z tej wtyczki.
 I2C do pinw I2C, MIRQ i "Radio on" do ktregokolwiek pinu.
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include "I2cbase.h"
#include "sagem_affa2.h"
#include "timer.h"
#include "usart.h"
#include "constDef.h"
#include "AVRTools/Analog2Digital.h"
#include <stdint.h>
#include <avr/wdt.h>

#include "temperature.h"
#include "voltage.h"
#include "digital.h"

void get_mcusr(void) __attribute__((naked))
__attribute__((section(".init3")));
void get_mcusr(void) {
	MCUSR = 0;
	wdt_disable();
}

#define TIMER0_COUNTER_START	105
volatile uint8_t timer1ms;
volatile uint16_t timer1s;
volatile uint32_t timer12s;
// Przerwanie timera 0; wywouje si co 0,1ms
ISR(TIMER0_OVF_vect)
{
	TCNT0 = TIMER0_COUNTER_START; //Reset licznika timera do pocztkowej wartoci
	++timer1ms;	//Zwikszenie licznika
	++timer1s;
	++timer12s;
}

uint8_t refreshDisplay = 0;

void displayText(const char *text)
{
	write_text_sagem(text, SCROLL_TEXT);
}

#define DELAYY _delay_ms(125)

uint8_t displayWatchdog = 0;

void serialPrint(const char *text)
{
	/* COMMENT IF DEBUG IS OFF
	printf(text);
	printf("\n");
	*/
}

char voltageText[5];
char temperatureText[3];
char textToDisplay[8];

void handleDisplay(int blink)
{	
	getVoltageText(voltageText);
	strcpy(textToDisplay, voltageText);
		
	getTemperatureText(temperatureText);
	strcat(textToDisplay, temperatureText);

	displayText(textToDisplay);
	
	handleDigital();

	/* COMMENT IF DEBUG IS OFF
	if(blink > 0)
	{
		sagem_affa2_set_icon(ICON_I_TRAFFIC);
	}
	else
	{
		sagem_affa2_clr_icon(ICON_I_TRAFFIC);
	}
	*/
}

void initMain()
{
	LED_PORT_CONF;
	LCD_RADIO_ON_CONF;
	LCD_RADIO_ON_ON;

	R2R_PORT_CONF;
	R2R_SET_0000;
	DDRD &= ~(1<<PD3);
	PORTD |= (1<<PD3);

	// ####### Ustawienia timerw ########
	// Timer 0; 8 bit		Bdzie suy do odmierzania czasu co 1ms (albo czciej)
	TCCR0B |= (1<<CS01);					// CS02 = 0; CS01 = 1; CS00 = 0; Wewntrzny zegar, preskaler 8 (str. 142 w nocie)
	TIMSK0 |= (1<<TOIE0);				// Uruchomienie przerwania po przepenieniu timera (str. 159)
	TCNT0 = TIMER0_COUNTER_START;		// Timer liczy od 105 do 255 co daje przerwanie rwno co 0,1ms

	sei();		//Globalne uruchomienie przerwa

	// ####### Ustawienie komunikacji RS232 z kompem ########
	//uart_set_FrameFormat(USART_8BIT_DATA|USART_1STOP_BIT|USART_NO_PARITY|USART_ASYNC_MODE);			// default settings
	
	/* COMMENT IF DEBUG IS OFF
	uart_init(BAUD_CALC(115200));				// 8n1 transmission is set as default
	stdout = &uart0_io;							// attach uart stream to stdout & stdin
	stdin = &uart0_io;							// uart0_in and uart0_out are only available if NO_USART_RX or NO_USART_TX is defined
	uart_putstr("Serial start at speed: 115200 kbps\n");
	*/

	initA2D(kA2dReference11V);			// Inicjalizacja ADC

	I2C_Init();				// Start I2C

	sagem_affa2_init();	// Inicjalizacja wywietlacza
	
	setTemperatureSensors();
	setDigital();
	
	wdt_enable(WDTO_8S);
}

void handle1ms(uint8_t uartBytes, uint8_t dataRead[6])
{
	wdt_reset();

	if(uart0_AvailableBytes() > 0)
	{
		_delay_ms(50);
		uartBytes = uart0_AvailableBytes();
		char buffer[uartBytes+1];
		uart_getln(buffer, uartBytes+1);
	}

	if(LCD_MIRQ_IS_SET)
	{
		LED_CLR;
	}
	else
	{
		LED_SET;
		read_sagem(dataRead);
		if(dataRead[0] == 0x01 && (dataRead[1] == 0x00 || dataRead[1] == 0x01)) // "Ping" from display
		{
			dataRead[1] = 0x11;		// Respond to keep communication
			write_sagem(dataRead);
			displayWatchdog = 0;
			serialPrint("Ping");
		}
	}
}

int main() 
{
	initMain();	
	uint8_t uartBytes, dataRead[6];

	int startScreen = 1; // to show welcome text
	//int startScreen = 0; // standard
	int blink = 1;
			
	while (1) 
	{
		if(timer1ms > 10) // Every ~1ms
		{		
			timer1ms = 1;
			timertick();
			handle1ms(uartBytes, dataRead);
		} 
		
		if(timer1s > 10000)	// Every ~1s
		{
			timer1s = 1;

			if(++displayWatchdog > 20)
			{
				sagem_affa2_init();
				timer12s = 1;
			}
		}
		
		//if(timer12s > 200000)// Every ~10s to show start text
		if(timer12s > 12000)	// Every ~1.2s
		{
			timer12s = 1;					
			handleDisplay(blink);
			blink = -blink;
		}
		
		/* UNCOMMENT to show start text
		if(startScreen > 0 && timer12s > 12000)
		{
			displayText("Regards to Renault Clio II fans ");
			startScreen = 0;
		}
		*/
	}
}

